@@ -405,12 +405,6 @@ nodoka_draw_entry (cairo_t * cr,
 	cairo_translate (cr, x + 0.5, y + 0.5);
 	cairo_set_line_width (cr, 1.0);
 
-	/* Fill the background (shouldn't have to) */
-	cairo_rectangle (cr, -0.5, -0.5, width, height);
-	cairo_set_source_rgb (cr, widget->parentbg.r, widget->parentbg.g,
-						  widget->parentbg.b);
-	cairo_fill (cr);
-
 	/* Fill the entry's base color */
 	nodoka_rounded_rectangle (cr, 0.5, 0.5, width - 2, height - 2, widget->roundness, widget->corners);
 	cairo_set_source_rgb (cr, base->r, base->g, base->b);
@@ -453,6 +447,7 @@ nodoka_draw_entry (cairo_t * cr,
 	}
 }
 
+#define TROUGH_SIZE 6
 void
 nodoka_draw_scale_trough (cairo_t * cr,
 						  const NodokaColors * colors,
@@ -460,77 +455,95 @@ nodoka_draw_scale_trough (cairo_t * cr,
 						  const SliderParameters * slider,
 						  int x, int y, int width, int height)
 {
-	int fill_x, fill_y, fill_width, fill_height;	
-	int trough_width, trough_height;
-	double translate_x, translate_y;
-	int fill_size = slider->fill_size;
-	int TROUGH_SIZE = 6;
+	int     trough_width, trough_height;
+	double  translate_x, translate_y;
+
+	cairo_save (cr);
 
 	if (slider->horizontal)
 	{
-		if (fill_size > width - 3)
-			fill_size = width - 3;
-
-		fill_x = slider->inverted ? width - fill_size - 3 : 0;
-		fill_y = 0;
-		fill_width = fill_size;
-		fill_height = TROUGH_SIZE - 2;
-
-		trough_width = width - 3;
-		trough_height = TROUGH_SIZE - 2;
-
-		translate_x = x + 0.5;
-		translate_y = y + 0.5 + (height / 2) - (TROUGH_SIZE / 2);
+		trough_width  = width;
+		trough_height = TROUGH_SIZE;
+		
+		translate_x   = x;
+		translate_y   = y + (height/2) - (TROUGH_SIZE/2);
 	}
 	else
 	{
-		if (fill_size > height - 3)
-			fill_size = height - 3;
-
-		fill_x = 0;
-		fill_y = slider->inverted ? height - fill_size - 3 : 0;
-		fill_width = TROUGH_SIZE - 2;
-		fill_height = fill_size;
-
-		trough_width = TROUGH_SIZE - 2;
-		trough_height = height - 3;
-
-		translate_x = x + 0.5 + (width / 2) - (TROUGH_SIZE / 2);
-		translate_y = y + 0.5;
+		trough_width  = TROUGH_SIZE;
+		trough_height = height;
+		
+		translate_x   = x + (width/2) - (TROUGH_SIZE/2);
+		translate_y  = y;
 	}
 
 	cairo_set_line_width (cr, 1.0);
 	cairo_translate (cr, translate_x, translate_y);
 
-	cairo_translate (cr, 1, 1);
+	/* Drawing Fill Level */
+	if (slider->fill_level)
+	{
+		NodokaRGB *fill = (NodokaRGB *) & colors->spot[1];
+		
+		cairo_rectangle (cr, 1.5, 1.5, trough_width - 3, trough_height - 3);
+		
+		nodoka_set_gradient (cr, fill, HOLLOW_BOTTOM_HILIGHT, 
+							HOLLOW_TOP_HILIGHT, GRADIENT_CENTER, 
+							(slider->horizontal ? 0 : trough_width),
+							(slider->horizontal ? trough_height : 0),
+							widget->gradients, TRUE, 0.35);
+		cairo_fill_preserve (cr);
+		
+		cairo_set_source_rgba (cr, colors->spot[2].r, colors->spot[2].g, 
+								colors->spot[2].b, 0.28);
+		cairo_stroke (cr);
+	}
+	/* Drawing Fill */
+	else if (slider->lower)
+	{
+		NodokaRGB *fill = (NodokaRGB *) & colors->spot[1];
+		
+		cairo_rectangle (cr, 1.5, 1.5, trough_width - 3, trough_height - 3);
+		
+		nodoka_set_gradient (cr, fill, HOLLOW_BOTTOM_HILIGHT, 
+							HOLLOW_TOP_HILIGHT, GRADIENT_CENTER, 
+							(slider->horizontal ? 0 : trough_width),
+							(slider->horizontal ? trough_height : 0),
+							widget->gradients, FALSE, 1.0);
+		cairo_fill_preserve (cr);
+		
+		cairo_set_source_rgba (cr, colors->spot[2].r, colors->spot[2].g, 
+								colors->spot[2].b, 0.8);
+		cairo_stroke (cr);
+	}
+	/* Drawing Trough */
+	else
+	{
+		NodokaRGB *bg = (NodokaRGB *) & colors->shade[3];
+		
+		cairo_rectangle (cr, 1.5, 1.5, trough_width - 3, trough_height - 3);
+		
+		nodoka_set_gradient (cr, bg, HOLLOW_BOTTOM_HILIGHT, 
+							HOLLOW_TOP_HILIGHT, GRADIENT_CENTER, 
+							(slider->horizontal ? 0 : trough_width),
+							(slider->horizontal ? trough_height : 0),
+							widget->gradients, FALSE, 1.0);
+		cairo_fill_preserve (cr);
+		
+		cairo_set_source_rgba (cr, colors->shade[5].r, colors->shade[5].g, 
+								colors->shade[5].b, 0.8);
+		cairo_stroke (cr);
+		
+		
+		/* Draw shadow  */
+		cairo_move_to (cr, 2.5, trough_height - 2.5);
+		cairo_line_to (cr, 2.5, 2.5);
+		cairo_line_to (cr, trough_width - 2.5, 2.5);
+		cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.1);
+		cairo_stroke (cr);
+	}
 
-	nodoka_scale_draw_gradient (cr, &colors->shade[3],	
-								&colors->shade[5],	
-								0, 0, trough_width, trough_height,
-								TRUE, slider->horizontal);
-
-	/* we want to draw the fill gradient using a direct gradient call */
-	nodoka_set_gradient (cr, &colors->spot[1], DARK_BOTTOM_HILIGHT,
-						  DARK_TOP_HILIGHT, GRADIENT_CENTER,
-						  (slider->horizontal ? 0 : fill_width),
-						  (slider->horizontal ? fill_height : 0), widget->gradients, FALSE,
-						  1.0);
-	cairo_rectangle (cr, fill_x + 0.5, fill_y + 0.5, fill_width - 1,
-					 fill_height - 1);
-	cairo_fill (cr);
-
-	/* Set the right border color */
-	cairo_rectangle (cr, fill_x, fill_y, fill_width, fill_height);
-	cairo_set_source_rgba (cr, colors->spot[2].r, colors->spot[2].g,
-						   colors->spot[2].b, 0.8);
-	cairo_stroke (cr);
-
-	/* Draw shadow */
-	cairo_move_to (cr, 1, trough_height - 2);
-	cairo_line_to (cr, 1, 1);
-	cairo_line_to (cr, trough_width - 2, 1);
-	cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.1);
-	cairo_stroke (cr);
+	cairo_restore (cr);
 }
 
 void
@@ -625,12 +638,6 @@ nodoka_draw_progressbar_trough (cairo_t * cr, const NodokaColors * colors,
 
 	cairo_translate (cr, x, y);
 
-	/* Fill the background (shouldn't have to) */
-	cairo_rectangle (cr, -0.5, -0.5, width + 1, height + 1);
-	cairo_set_source_rgb (cr, widget->parentbg.r, widget->parentbg.g,
-						  widget->parentbg.b);
-	cairo_fill (cr);
-
 	cairo_set_line_width (cr, 1.0);
 
 
@@ -747,6 +754,78 @@ nodoka_draw_progressbar_fill (cairo_t * cr, const NodokaColors * colors,
 	cairo_stroke (cr);
 }
 
+void nodoka_draw_entry_progress (cairo_t * cr,
+						const NodokaColors * colors,
+						const WidgetParameters * widget,
+						const EntryProgressParameters * progress, 
+						int x, int y, int width, int height)
+{
+	NodokaRGB border;
+	NodokaRGB fill;
+	gint entry_width, entry_height;
+	double entry_radius;
+	double radius;
+	
+	cairo_save (cr);
+
+	fill = colors->bg[widget->state_type];
+	nodoka_shade (&fill, &border, 0.9);
+
+	if (progress->max_size_known)
+	{
+		entry_width = progress->max_size.width + progress->border.left + progress->border.right;
+		entry_height = progress->max_size.height + progress->border.top + progress->border.bottom;
+		entry_radius = MIN (widget->roundness, MIN ((entry_width - 4.0) / 2.0, (entry_height - 4.0) / 2.0));
+	}
+	else
+	{
+		entry_radius = widget->roundness;
+	}
+
+	radius = MAX (0, entry_radius + 1.0 - MAX (MAX (progress->border.left, progress->border.right),
+	                                           MAX (progress->border.top, progress->border.bottom)));
+
+	if (progress->max_size_known)
+	{
+		/* Clip to the max size, and then draw a (larger) rectangle ... */
+		nodoka_rounded_rectangle (cr, progress->max_size.x,
+		                              progress->max_size.y,
+		                              progress->max_size.width,
+		                              progress->max_size.height,
+		                              radius,
+		                              NDK_CORNER_ALL);
+		cairo_clip (cr);
+
+		/* We just draw wider by one pixel ... */
+		cairo_set_source_rgb (cr, fill.r, fill.g, fill.b);
+		cairo_rectangle (cr, x, y + 1, width, height - 2);
+		cairo_fill (cr);
+
+		cairo_set_line_width (cr, 1.0);
+		cairo_set_source_rgb (cr, border.r, border.g, border.b);
+		cairo_rectangle (cr, x - 0.5, y + 0.5, width + 1, height - 1);
+		cairo_stroke (cr);
+	}
+	else
+	{
+		nodoka_rounded_rectangle (cr, x, y, width + 10, height + 10, radius, NDK_CORNER_ALL);
+		cairo_clip (cr);
+		nodoka_rounded_rectangle (cr, x - 10, y - 10, width + 10, height + 10, radius, NDK_CORNER_ALL);
+		cairo_clip (cr);
+
+		cairo_set_source_rgb (cr, fill.r, fill.g, fill.b);
+		nodoka_rounded_rectangle (cr, x + 1, y + 1, width - 2, height - 2, radius, NDK_CORNER_ALL);
+		cairo_fill (cr);
+
+		cairo_set_line_width (cr, 1.0);
+		cairo_set_source_rgb (cr, border.r, border.g, border.b);
+		nodoka_rounded_rectangle (cr, x + 0.5, y + 0.5, width - 1.0, height - 1.0, radius, NDK_CORNER_ALL);
+		cairo_stroke (cr);
+	}
+
+	cairo_restore (cr);	
+}							
+
 void
 nodoka_draw_optionmenu (cairo_t * cr,
 						const NodokaColors * colors,
@@ -1708,6 +1787,15 @@ nodoka_draw_handle (cairo_t * cr, const NodokaColors * colors,
 					int height)
 {
 	NodokaRGB *dark = (NodokaRGB *) & colors->shade[3];
+	
+	NodokaRGB *bg = (NodokaRGB *) & colors->bg[widget->state_type];
+	
+	if (handle->type == NDK_HANDLE_SPLITTER)
+	{
+		cairo_set_source_rgb (cr, bg->r, bg->g, bg->b);
+		cairo_rectangle (cr, x, y, width, height);
+		cairo_fill (cr);
+	}
 
 	int bar_height;
 	int i;
@@ -1901,7 +1989,7 @@ nodoka_draw_radiobutton (cairo_t * cr, const NodokaColors * colors,
 
 	int radius = ((width < height) ? width / 2 : height / 2);
 
-	if (widget->state_type == GTK_STATE_INSENSITIVE)
+	if (widget->state_type == NDK_STATE_INSENSITIVE)
 	{
 		border = colors->shade[3];
 		dot = colors->shade[3];
@@ -1918,7 +2006,7 @@ nodoka_draw_radiobutton (cairo_t * cr, const NodokaColors * colors,
 
 	/* Draw backround */
 	cairo_set_line_width (cr, 1);
-	if (widget->state_type != GTK_STATE_INSENSITIVE)
+	if (widget->state_type != NDK_STATE_INSENSITIVE)
 	{
 		NodokaRGB bg = colors->base[0];
 		if (widget->prelight)
@@ -1994,7 +2082,7 @@ nodoka_draw_checkbutton (cairo_t * cr,
 		width = height;
 	}
 	
-	if (widget->state_type == GTK_STATE_INSENSITIVE)
+	if (widget->state_type == NDK_STATE_INSENSITIVE)
 	{
 		border = colors->shade[3];
 		dot = colors->shade[3];
@@ -2033,7 +2121,7 @@ nodoka_draw_checkbutton (cairo_t * cr,
 	}
 
 
-	if (widget->state_type != GTK_STATE_INSENSITIVE)
+	if (widget->state_type != NDK_STATE_INSENSITIVE)
 	{
 		NodokaRGB bg = colors->base[0];
 		if (widget->prelight)
