@@ -83,8 +83,8 @@ nodoka_set_widget_parameters (const GtkWidget * widget,
 							  const GtkStyle * style, GtkStateType state_type,
 							  WidgetParameters * params)
 {
-	if (widget && GTK_IS_ENTRY (widget))
-		state_type = GTK_WIDGET_STATE (widget);
+	if (state_type == GTK_STATE_NORMAL && widget && GTK_IS_ENTRY (widget))
+		state_type = gtk_widget_get_state ((GtkWidget *) widget);
 
 	params->active = (state_type == GTK_STATE_ACTIVE);
 	params->prelight = (state_type == GTK_STATE_PRELIGHT);
@@ -94,10 +94,13 @@ nodoka_set_widget_parameters (const GtkWidget * widget,
 	params->roundness = NODOKA_STYLE (style)->roundness;
 	params->hilight_ratio = NODOKA_STYLE (style)->hilight_ratio;
 	params->gradients = NODOKA_STYLE (style)->gradients;
-	params->ltr = !(nodoka_get_direction ((GtkWidget *) widget) == GTK_TEXT_DIR_RTL);
+	if (widget)
+		params->ltr = !(nodoka_get_direction ((GtkWidget *) widget) == GTK_TEXT_DIR_RTL);
+	else
+		params->ltr = TRUE;
 
-	params->focus = widget && GTK_WIDGET_HAS_FOCUS (widget);
-	params->is_default = widget && GTK_WIDGET_HAS_DEFAULT (widget);
+	params->focus = widget && gtk_widget_has_focus ((GtkWidget *) widget);
+	params->is_default = widget && gtk_widget_has_default ((GtkWidget *) widget);
 
 	if (!params->active && widget && GTK_IS_TOGGLE_BUTTON (widget))
 		params->active =
@@ -147,7 +150,7 @@ nodoka_style_draw_flat_box (DRAW_ARGS)
 
 		cairo_destroy (cr);
 	}
-	else if (DETAIL ("entry_bg") && !(widget && widget->parent && GTK_IS_TREE_VIEW (widget->parent)))
+	else if (DETAIL ("entry_bg") && !(widget && gtk_widget_get_parent (widget) && GTK_IS_TREE_VIEW (gtk_widget_get_parent (widget))))
 	{
 		/* This is a workaround to allow big roundness in entries  *
 		 * which are drown twice and so parts of rounded edges get *
@@ -169,9 +172,9 @@ nodoka_style_draw_flat_box (DRAW_ARGS)
 		width += style->xthickness*2;
 		height += style->ythickness*2;
 
-		if (widget && widget->parent
-			&& (GTK_IS_COMBO (widget->parent)
-				|| GTK_IS_COMBO_BOX_ENTRY (widget->parent)
+		if (widget && gtk_widget_get_parent (widget)
+			&& (NDK_IS_COMBO (gtk_widget_get_parent (widget))
+				|| GTK_IS_COMBO_BOX_ENTRY (gtk_widget_get_parent (widget))
 				|| GTK_IS_SPIN_BUTTON (widget)))
 		{
 			width += style->xthickness;
@@ -184,7 +187,7 @@ nodoka_style_draw_flat_box (DRAW_ARGS)
 			else
 				params.corners = NDK_CORNER_TOPLEFT | NDK_CORNER_BOTTOMLEFT;
 
-			if (GTK_IS_COMBO_BOX_ENTRY (widget->parent))
+			if (GTK_IS_COMBO_BOX_ENTRY (gtk_widget_get_parent (widget)))
 			{
 				height += 1;
 				/* GTKComboBoxEntry workaround */
@@ -264,15 +267,15 @@ nodoka_style_draw_shadow (DRAW_ARGS)
 	nodoka_sanitize_size (window, &width, &height);
 
 	if (DETAIL ("entry")
-		&& !(widget && widget->parent && GTK_IS_TREE_VIEW (widget->parent)))
+		&& !(widget && gtk_widget_get_parent (widget) && GTK_IS_TREE_VIEW (gtk_widget_get_parent (widget))))
 	{
 		WidgetParameters params;
 
 		nodoka_set_widget_parameters (widget, style, state_type, &params);
 
-		if (widget && widget->parent
-			&& (GTK_IS_COMBO (widget->parent)
-				|| GTK_IS_COMBO_BOX_ENTRY (widget->parent)
+		if (widget && gtk_widget_get_parent (widget)
+			&& (NDK_IS_COMBO (gtk_widget_get_parent (widget))
+				|| GTK_IS_COMBO_BOX_ENTRY (gtk_widget_get_parent (widget))
 				|| GTK_IS_SPIN_BUTTON (widget)))
 		{
 			width += style->xthickness;
@@ -294,10 +297,17 @@ nodoka_style_draw_shadow (DRAW_ARGS)
 		if (params.roundness <= 0)
 			params.corners = NDK_CORNER_NONE;
 
+                if (!widget || !g_object_get_data (G_OBJECT (widget), "transparent-bg-hint"))
+                {
+                	cairo_rectangle(cr, 0, 0, width, height);
+                	cairo_set_source_rgb(cr, params.parentbg.r, params.parentbg.g, params.parentbg.b);
+            	        cairo_fill(cr);
+                }
+                
 		nodoka_draw_entry (cr, &nodoka_style->colors, &params, &entry, x, y, width,
 						   height);
 	}
-	else if (DETAIL ("frame") && widget && widget->parent && GTK_IS_STATUSBAR (widget->parent))
+	else if (DETAIL ("frame") && widget && gtk_widget_get_parent (widget) && GTK_IS_STATUSBAR (gtk_widget_get_parent (widget)))
 	{
 		WidgetParameters params;
 
@@ -364,7 +374,7 @@ nodoka_style_draw_box_gap (DRAW_ARGS, GtkPositionType gap_side, gint gap_x,
 
 		/* Somehow needed to fix full-width notebook problem */
 		gint window_width;
-		gdk_window_get_size (window, &window_width, NULL);
+		gdk_drawable_get_size (window, &window_width, NULL);
 		if ((nodoka_style->roundness == 0) || window_width == width)
 		{
 			params.corners = NDK_CORNER_NONE;
@@ -377,10 +387,12 @@ nodoka_style_draw_box_gap (DRAW_ARGS, GtkPositionType gap_side, gint gap_x,
 					NDK_CORNER_BOTTOMRIGHT | NDK_CORNER_BOTTOMLEFT;
 
 				if (frame.gap_width != width)
+				{
 					if (params.ltr) 
 						params.corners |= NDK_CORNER_TOPRIGHT;
 					else
 						params.corners |= NDK_CORNER_TOPLEFT;
+				}
 			}
 			else if (frame.gap_side == NDK_GAP_BOTTOM)
 			{
@@ -388,10 +400,12 @@ nodoka_style_draw_box_gap (DRAW_ARGS, GtkPositionType gap_side, gint gap_x,
 					NDK_CORNER_TOPRIGHT | NDK_CORNER_TOPLEFT;
 
 				if (frame.gap_width != width)
+				{
 					if (params.ltr) 
 						params.corners |= NDK_CORNER_BOTTOMRIGHT;
 					else
 						params.corners |= NDK_CORNER_BOTTOMLEFT;
+				}
 			}
 			else if (frame.gap_side == NDK_GAP_LEFT)
 			{
@@ -554,14 +568,14 @@ nodoka_style_draw_box (DRAW_ARGS)
 	cairo_t *cr = nodoka_begin_paint (window, area);
 
 	if ((width == -1) && (height == -1))
-		gdk_window_get_size (window, &width, &height);
+		gdk_drawable_get_size (window, &width, &height);
 	else if (width == -1)
-		gdk_window_get_size (window, &width, NULL);
+		gdk_drawable_get_size (window, &width, NULL);
 	else if (height == -1)
-		gdk_window_get_size (window, NULL, &height);
+		gdk_drawable_get_size (window, NULL, &height);
 
 	if (DETAIL ("menubar")
-		&& !(widget && widget->parent && (ndk_is_panel_widget (widget->parent))))
+		&& !(widget && gtk_widget_get_parent (widget) && (ndk_is_panel_widget (gtk_widget_get_parent (widget)))))
 	{
 		WidgetParameters params;
 
@@ -572,11 +586,11 @@ nodoka_style_draw_box (DRAW_ARGS)
 
 		nodoka_draw_menubar (cr, colors, &params, &menubar, x, y, width, height);
 	}
-	else if (DETAIL ("button") && widget && widget->parent
-			 && (GTK_IS_TREE_VIEW (widget->parent)
-				 || GTK_IS_CLIST (widget->parent) 
-				 || NDK_IS_ETREE (widget->parent)
-				 || NDK_IS_ETABLE (widget->parent)))
+	else if (DETAIL ("button") && widget && gtk_widget_get_parent (widget)
+			 && (GTK_IS_TREE_VIEW (gtk_widget_get_parent (widget))
+			 	 || NDK_IS_CLIST (gtk_widget_get_parent (widget))
+				 || NDK_IS_ETREE (gtk_widget_get_parent (widget))
+				 || NDK_IS_ETABLE (gtk_widget_get_parent (widget))))
 	{
 		WidgetParameters params;
 		ListViewHeaderParameters header;
@@ -588,19 +602,21 @@ nodoka_style_draw_box (DRAW_ARGS)
 		nodoka_set_widget_parameters (widget, style, state_type, &params);
 		params.corners = NDK_CORNER_NONE;
 
-		if (widget && widget->parent && GTK_IS_TREE_VIEW (widget->parent))
+		if (widget && gtk_widget_get_parent (widget) && GTK_IS_TREE_VIEW (gtk_widget_get_parent (widget)))
 		{
 			nodoka_gtk_treeview_get_header_index (GTK_TREE_VIEW
-						  (widget->parent), widget,
+						  (gtk_widget_get_parent (widget)), widget,
 						  &column_index, &columns,
 						  &resizable);
 		}
-		else if (widget && widget->parent && GTK_IS_CLIST (widget->parent))
+#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 90 && !defined(GTK_DISABLE_DEPRECATED)
+		else if (widget && gtk_widget_get_parent (widget) && NDK_IS_CLIST (gtk_widget_get_parent (widget)))
 		{
-			nodoka_gtk_clist_get_header_index (GTK_CLIST (widget->parent),
+			nodoka_gtk_clist_get_header_index (GTK_CLIST (gtk_widget_get_parent (widget)),
 						   widget, &column_index, &columns);
 		}
-
+#endif
+ 
 		header.resizable = resizable;
 
 		if (column_index == 0)
@@ -629,9 +645,9 @@ nodoka_style_draw_box (DRAW_ARGS)
 			horizontal = FALSE;
 
 		boolean shadows = TRUE;
-		if ((widget && widget->parent
-			 && (GTK_IS_COMBO_BOX_ENTRY (widget->parent)
-				 || GTK_IS_COMBO (widget->parent))))
+		if ((widget && gtk_widget_get_parent (widget)
+			 && (NDK_IS_COMBO (gtk_widget_get_parent (widget))
+				|| GTK_IS_COMBO_BOX_ENTRY (gtk_widget_get_parent (widget)))))
 		{
 
 			if (nodoka_style->roundness > 0)
@@ -684,7 +700,7 @@ nodoka_style_draw_box (DRAW_ARGS)
 
 
 	  	/* Only draw button as disabled if entire widget is disabled */
-	  	if (params.disabled && !(GTK_WIDGET_STATE(widget) == GTK_STATE_INSENSITIVE)) 
+	  	if (params.disabled && !(gtk_widget_get_state ((GtkWidget *) widget) == GTK_STATE_INSENSITIVE)) 
 		{
 			params.disabled = FALSE;
 			params.state_type = GTK_STATE_NORMAL;
@@ -728,37 +744,23 @@ nodoka_style_draw_box (DRAW_ARGS)
 			
 	}
 	else if (DETAIL ("spinbutton"))	{ }
-	else if (DETAIL ("trough") && widget && GTK_IS_SCALE (widget))
+	else if (detail && g_str_has_prefix (detail, "trough") && widget && GTK_IS_SCALE (widget))
 	{
-		GtkAdjustment *adjustment =
-			gtk_range_get_adjustment (GTK_RANGE (widget));
 		WidgetParameters params;
 		SliderParameters slider;
-		gint slider_length;
 
-		gtk_widget_style_get (widget, "slider-length", &slider_length, NULL);
 		nodoka_set_widget_parameters (widget, style, state_type, &params);
 		params.corners = NDK_CORNER_NONE;
 
-		slider.inverted = gtk_range_get_inverted (GTK_RANGE (widget));
+		slider.lower = DETAIL ("trough-lower");
+		slider.fill_level = DETAIL ("trough-fill-level") || DETAIL ("trough-fill-level-full");
+
 		slider.horizontal =
-			(GTK_RANGE (widget)->orientation == GTK_ORIENTATION_HORIZONTAL);
-		if ((adjustment->upper - adjustment->page_size - 
-                     adjustment->lower) != 0)
-			slider.fill_size = 
-				((slider.horizontal ? width : height) - 
-                                 slider_length) * 
-				((adjustment->value  - adjustment->lower) / 
-                                 (adjustment->upper - adjustment->page_size - 
-                                  adjustment->lower)) + slider_length / 2;
-		else
-			slider.fill_size = 0;
-		if (slider.horizontal)
-			slider.inverted =
-				slider.inverted != (nodoka_get_direction (widget) ==
-									GTK_TEXT_DIR_RTL);
+			(GTK_IS_HSCALE (widget));
+
 		slider.focus.inner = nodoka_style->focus_inner;
 		slider.focus.fill = nodoka_style->focus_fill;
+
 		nodoka_draw_scale_trough (cr, &nodoka_style->colors, &params, &slider,
 								  x, y, width, height);
 	}
@@ -777,6 +779,13 @@ nodoka_style_draw_box (DRAW_ARGS)
 		else
 			params.corners = NDK_CORNER_NONE;
 
+                if (!widget || !g_object_get_data (G_OBJECT (widget), "transparent-bg-hint"))
+                {
+                	cairo_rectangle(cr, 0, 0, width, height);
+                	cairo_set_source_rgb(cr, params.parentbg.r, params.parentbg.g, params.parentbg.b);
+            	        cairo_fill(cr);
+                }
+
 		nodoka_draw_progressbar_trough (cr, colors, &params, &progressbar, x, y, width,
 										height);
 	}
@@ -785,24 +794,23 @@ nodoka_style_draw_box (DRAW_ARGS)
 	{
 		WidgetParameters params;
 		ScrollBarParameters scrollbar;
+		GtkAllocation allocation;
+
+		gtk_widget_get_allocation (widget, &allocation);
 
 		nodoka_set_widget_parameters (widget, style, state_type, &params);
 
 		/* Workaround some gecko < 1.9 crap */
-		if ((widget->allocation.x == -1) || (widget->allocation.y == -1))
+		if ((allocation.x == -1) || (allocation.y == -1))
 			params.roundness = 0;
 
 		if (params.roundness > 0)
 			params.roundness = 6;
 
-		scrollbar.horizontal = TRUE;
+		scrollbar.horizontal = GTK_IS_HSCROLLBAR (widget);
 		scrollbar.junction = nodoka_scrollbar_get_junction (widget);
 		scrollbar.steppers = nodoka_scrollbar_visible_steppers (widget);
 
-		if (GTK_IS_RANGE (widget))
-			scrollbar.horizontal =
-				GTK_RANGE (widget)->orientation == GTK_ORIENTATION_HORIZONTAL;
-
 		params.corners = NDK_CORNER_NONE;
 		if (params.roundness > 0)
 		{
@@ -950,6 +958,132 @@ nodoka_style_draw_box (DRAW_ARGS)
 #endif
 		nodoka_draw_progressbar_fill (cr, colors, &params, &progressbar, x - 1, y, width + 2, height);
 	}
+	else if (DETAIL ("entry-progress"))
+	{
+		WidgetParameters params;
+		EntryProgressParameters progress;
+
+		nodoka_set_widget_parameters (widget, style, state_type, &params);
+
+		progress.max_size_known = FALSE;
+		progress.max_size.x = 0;
+		progress.max_size.y = 0;
+		progress.max_size.width = 0;
+		progress.max_size.height = 0;
+		progress.border.left = style->xthickness;
+		progress.border.right = style->xthickness;
+		progress.border.top = style->ythickness;
+		progress.border.bottom = style->ythickness;
+
+		if (widget && GTK_IS_ENTRY (widget))
+		{
+			GtkBorder *border;
+			/* Try to retrieve the style property. */
+			gtk_widget_style_get (widget,
+			                      "progress-border", &border,
+			                      NULL);
+
+			if (border)
+			{
+				progress.border = *border;
+				gtk_border_free (border);
+			}
+
+			/* We got an entry, but well, we may not be drawing to
+			 * this particular widget ... it may not even be realized.
+			 * Also, we need to be drawing on a window obviously ... */
+#if (GTK_MAJOR_VERSION >= 3) || (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 20)
+			if (gtk_widget_get_realized (widget) &&
+#else
+			if (GTK_WIDGET_REALIZED (widget) &&
+#endif
+			    window && GDK_IS_WINDOW (window) &&
+			    gdk_window_is_visible (gtk_widget_get_window (widget)))
+			{
+				/* Assumptions done by this code:
+				 *  - GtkEntry has some nested windows.
+				 *  - widget->window is the entries window
+				 *  - widget->window is the size of the entry part
+				 *    (and not larger)
+				 *  - only one layer of subwindows
+				 * These should be true with any GTK+ 2.x version.
+				 */
+
+				if (gtk_widget_get_window (widget) == window)
+				{
+					progress.max_size_known = TRUE;
+					gdk_drawable_get_size (gtk_widget_get_window (widget),
+					                       &progress.max_size.width,
+					                       &progress.max_size.height);
+
+				}
+				else
+				{
+					GdkWindow *parent;
+					parent = gdk_window_get_parent (window);
+					if (gtk_widget_get_window (widget) == parent)
+					{
+						gint pos_x, pos_y;
+						/* widget->window is the parent window
+						 * of the current one. This means we can
+						 * calculate the correct offsets. */
+						gdk_window_get_position (window, &pos_x, &pos_y);
+						progress.max_size.x = -pos_x;
+						progress.max_size.y = -pos_y;
+
+						progress.max_size_known = TRUE;
+						gdk_drawable_get_size (gtk_widget_get_window (widget),
+						                       &progress.max_size.width,
+						                       &progress.max_size.height);
+					} /* Nothing we can do in this case ... */
+				}
+
+				/* Now, one more thing needs to be done. If interior-focus
+				 * is off, then the entry may be a bit smaller. */
+				if (progress.max_size_known && gtk_widget_has_focus (widget))
+				{
+					gboolean interior_focus = TRUE;
+					gint focus_line_width = 1;
+
+					gtk_widget_style_get (widget,
+					                      "interior-focus", &interior_focus,
+					                      "focus-line-width", &focus_line_width,
+					                      NULL);
+
+					if (!interior_focus)
+					{
+						progress.max_size.x += focus_line_width;
+						progress.max_size.y += focus_line_width;
+						progress.max_size.width -= 2*focus_line_width;
+						progress.max_size.height -= 2*focus_line_width;
+					}
+				}
+
+				if (progress.max_size_known)
+				{
+					progress.max_size.x += progress.border.left;
+					progress.max_size.y += progress.border.top;
+					progress.max_size.width -= progress.border.left + progress.border.right;
+					progress.max_size.height -= progress.border.top + progress.border.bottom;
+
+					/* Now test that max_size.height == height, if that
+					 * fails, something has gone wrong ... so then throw away
+					 * the max_size information. */
+					if (progress.max_size.height != height)
+					{
+						progress.max_size_known = FALSE;
+						progress.max_size.x = 0;
+						progress.max_size.y = 0;
+						progress.max_size.width = 0;
+						progress.max_size.height = 0;
+					}
+				}
+			}
+		}
+		
+		nodoka_draw_entry_progress (cr, colors, &params, &progress, x - 1, y, width + 2, height);
+
+	}
 	else if (DETAIL ("hscale") || DETAIL ("vscale"))
 	{
 		WidgetParameters params;
@@ -966,8 +1100,8 @@ nodoka_style_draw_box (DRAW_ARGS)
 			params.corners = NDK_CORNER_NONE;
 
 		SliderParameters slider;
-		slider.inverted = FALSE;
-		slider.fill_size = 0;
+		//slider.inverted = FALSE;
+		//slider.fill_size = 0;
 		slider.horizontal = horizontal;
 		slider.focus.inner = nodoka_style->focus_inner;
 		slider.focus.fill = nodoka_style->focus_fill;
@@ -1015,12 +1149,12 @@ nodoka_style_draw_box (DRAW_ARGS)
 
 		params.corners = NDK_CORNER_NONE;
 
-		if (widget && widget->parent && !(GTK_IS_MENU_BAR (widget->parent)))
+		if (widget && gtk_widget_get_parent (widget) && !(GTK_IS_MENU_BAR (gtk_widget_get_parent (widget))))
 		{
 			nodoka_draw_menuitem (cr, colors, &params, x, y, width, height);
 		}
 
-		if (widget && widget->parent && GTK_IS_MENU_BAR (widget->parent))
+		if (widget && gtk_widget_get_parent (widget) && GTK_IS_MENU_BAR (gtk_widget_get_parent (widget)))
 		{
 			params.active = TRUE;
 			params.prelight = TRUE;
@@ -1043,26 +1177,24 @@ nodoka_style_draw_box (DRAW_ARGS)
 		WidgetParameters params;
 		ScrollBarParameters scrollbar;
 
+		GtkAllocation allocation;
+		gtk_widget_get_allocation (widget, &allocation);
+
 		nodoka_set_widget_parameters (widget, style, state_type, &params);
 		
 		/* Workaround some gecko < 1.9 crap */
-		if ((widget->allocation.x == -1) || (widget->allocation.y == -1))
+		if ((allocation.x == -1) || (allocation.y == -1))
 			params.roundness = 0;
 		
 		if (params.roundness > 0)
 			params.roundness = 6;
 		params.corners = NDK_CORNER_NONE;
 		scrollbar.has_color = FALSE;
-		scrollbar.horizontal = TRUE;
+		scrollbar.horizontal = GTK_IS_HSCROLLBAR (widget);
 		scrollbar.junction = nodoka_scrollbar_get_junction (widget);
 		scrollbar.steppers = nodoka_scrollbar_visible_steppers (widget);
 		scrollbar.style = nodoka_style->scrollbarstyle;
-
-		if (widget && GTK_IS_RANGE (widget))
-			scrollbar.horizontal =
-				GTK_RANGE (widget)->orientation == GTK_ORIENTATION_HORIZONTAL;
-
-		
+	
 		if (params.roundness > 0)
 		{
 			if (scrollbar.horizontal)
@@ -1215,10 +1347,10 @@ nodoka_style_draw_option (DRAW_ARGS)
 	if (!(GTK_IS_CHECK_BUTTON (widget)))
 		params.focus = FALSE;
 
-	if (DETAIL ("cellradio") && widget && widget->parent)
+	if (DETAIL ("cellradio") && !params.disabled && widget && gtk_widget_get_parent (widget))
 	{
-		params.disabled = (GTK_WIDGET_STATE (widget->parent) == GTK_STATE_INSENSITIVE);
-		params.state_type = GTK_WIDGET_STATE (widget->parent);
+		params.disabled = (gtk_widget_get_state (gtk_widget_get_parent (widget)) == GTK_STATE_INSENSITIVE);
+		params.state_type = gtk_widget_get_state (gtk_widget_get_parent (widget));
 	}
 	/* --- */
 
@@ -1268,10 +1400,10 @@ nodoka_style_draw_check (DRAW_ARGS)
 	if (!(GTK_IS_CHECK_BUTTON (widget)))
 		params.focus = FALSE;
 
-	if (DETAIL ("cellcheck") && widget && widget->parent)
+	if (DETAIL ("cellcheck") && !params.disabled && widget && gtk_widget_get_parent (widget))
 	{
-		params.disabled = (GTK_WIDGET_STATE (widget->parent) == GTK_STATE_INSENSITIVE);
-		params.state_type = GTK_WIDGET_STATE (widget->parent);
+		params.disabled = (gtk_widget_get_state (gtk_widget_get_parent (widget)) == GTK_STATE_INSENSITIVE);
+		params.state_type = gtk_widget_get_state (gtk_widget_get_parent (widget));
 	}
 	/* --- */
 
@@ -1333,9 +1465,9 @@ nodoka_style_draw_vline (GtkStyle * style, GdkWindow * window,
 	NodokaColors *colors = &nodoka_style->colors;
 
 	if (!
-		(widget && widget->parent && GTK_IS_HBOX (widget->parent) && widget->parent->parent
-		 && GTK_IS_TOGGLE_BUTTON (widget->parent->parent) && widget->parent->parent->parent
-		 && GTK_IS_COMBO_BOX (widget->parent->parent->parent)))
+		(widget && gtk_widget_get_parent (widget) && GTK_IS_HBOX (gtk_widget_get_parent (widget)) && gtk_widget_get_parent (gtk_widget_get_parent (widget))
+		 && GTK_IS_TOGGLE_BUTTON (gtk_widget_get_parent (gtk_widget_get_parent (widget))) && gtk_widget_get_parent (gtk_widget_get_parent (gtk_widget_get_parent (widget)))
+		 && GTK_IS_COMBO_BOX (gtk_widget_get_parent (gtk_widget_get_parent (gtk_widget_get_parent (widget))))))
 	{
 		nodoka_draw_separator (cr, colors, NULL, &separator, x, y1, 2, y2 - y1);
 	}
@@ -1459,9 +1591,9 @@ nodoka_style_draw_arrow (GtkStyle * style, GdkWindow * window,
 		return;
 	}
 
-	if (widget && widget->parent && widget->parent->parent && widget->parent->parent->parent 
-	    && GTK_IS_COMBO_BOX (widget->parent->parent->parent) && 
-	    !(GTK_IS_COMBO_BOX_ENTRY (widget->parent->parent->parent)))
+	if (widget && gtk_widget_get_parent (widget) && gtk_widget_get_parent (gtk_widget_get_parent (widget)) && gtk_widget_get_parent (gtk_widget_get_parent (gtk_widget_get_parent (widget))) 
+	    && GTK_IS_COMBO_BOX (gtk_widget_get_parent (gtk_widget_get_parent (gtk_widget_get_parent (widget)))) && 
+	    !(GTK_IS_COMBO_BOX_ENTRY (gtk_widget_get_parent (gtk_widget_get_parent (gtk_widget_get_parent (widget))))))
 		x += 1;
 
 
@@ -1521,10 +1653,10 @@ nodoka_style_draw_layout (GtkStyle * style, GdkWindow * window,
 		GdkColor etched;
 		NodokaRGB temp;
 
-		if (widget && GTK_WIDGET_NO_WINDOW (widget))
+		if (widget && !gtk_widget_get_has_window (widget))
 			nodoka_shade (&params.parentbg, &temp, 1.2);
 		else
-			nodoka_shade (&colors->bg[widget->state], &temp, 1.2);
+			nodoka_shade (&colors->bg[gtk_widget_get_state (widget)], &temp, 1.2);
 		etched.red = (int) (temp.r * 65535);
 		etched.green = (int) (temp.g * 65535);
 		etched.blue = (int) (temp.b * 65535);
@@ -1678,9 +1810,8 @@ nodoka_style_draw_focus (GtkStyle * style, GdkWindow * window,
 			WidgetParameters params;
 			nodoka_set_widget_parameters (widget, style, state_type, &params);
 
-			if ((widget && widget->parent
-				 && (GTK_IS_COMBO_BOX_ENTRY (widget->parent)
-					 || GTK_IS_COMBO (widget->parent))))
+			if ((widget && gtk_widget_get_parent (widget)
+				 && (GTK_IS_COMBO_BOX_ENTRY (gtk_widget_get_parent (widget)))))
 			{
 	
 				if (nodoka_style->roundness > 0)
@@ -1715,8 +1846,8 @@ nodoka_style_draw_focus (GtkStyle * style, GdkWindow * window,
 			nodoka_draw_focus (cr, &nodoka_style->colors, &params, &focus, x - padding, y - padding, width + padding*2, height + padding*2);
 		}
 	}
-	/* Handle Tree View and CList */
-	else if (widget && (GTK_IS_TREE_VIEW (widget) || GTK_IS_CLIST (widget)))
+	/* Handle Tree View */
+	else if (widget && (GTK_IS_TREE_VIEW (widget) || NDK_IS_CLIST (widget)))
 	{
 		cairo_rectangle (cr, x + 0.5, y + 0.5, width - 1, height - 1);
 		cairo_set_line_width (cr, 1);
@@ -1739,7 +1870,7 @@ nodoka_style_draw_focus (GtkStyle * style, GdkWindow * window,
 	}
 	/* And now handle the rest */
 	else if (widget && !((GTK_IS_BUTTON (widget) || GTK_IS_RANGE (widget) || GTK_IS_CHECK_BUTTON (widget) 
-                              || GTK_IS_NOTEBOOK (widget)) && GTK_WIDGET_HAS_FOCUS (widget)))
+                              || GTK_IS_NOTEBOOK (widget)) && gtk_widget_has_focus ((GtkWidget *) widget)))
 	{
 		WidgetParameters params;
 		nodoka_set_widget_parameters (widget, style, state_type, &params);
@@ -1751,9 +1882,9 @@ nodoka_style_draw_focus (GtkStyle * style, GdkWindow * window,
 		if (widget && GTK_IS_BUTTON (widget))
 		{
 			padding += 2;
-			if ((widget && widget->parent
-				 && (GTK_IS_COMBO_BOX_ENTRY (widget->parent)
-					 || GTK_IS_COMBO (widget->parent))))
+			if ((widget && gtk_widget_get_parent (widget)
+				 && (NDK_IS_COMBO (gtk_widget_get_parent (widget))
+					|| GTK_IS_COMBO_BOX_ENTRY (gtk_widget_get_parent (widget)))))
 			{
 		
 				if (nodoka_style->roundness > 0)
